@@ -10,7 +10,6 @@ import numpy as np
 
 import pytest
 from tlz import valmap, first
-from tornado import gen
 from tornado.ioloop import IOLoop
 
 import dask
@@ -29,7 +28,8 @@ from distributed.utils_test import (  # noqa: F401
 )
 
 
-@gen_cluster(nthreads=[])
+# FIXME why does this leave behind unclosed Comm objects?
+@gen_cluster(nthreads=[], allow_unclosed=True)
 async def test_nanny(s):
     async with Nanny(s.address, nthreads=2, loop=s.loop) as n:
         async with rpc(n.address) as nn:
@@ -60,16 +60,16 @@ async def test_nanny(s):
 
 
 @gen_cluster(nthreads=[])
-def test_many_kills(s):
-    n = yield Nanny(s.address, nthreads=2, loop=s.loop)
+async def test_many_kills(s):
+    n = await Nanny(s.address, nthreads=2, loop=s.loop)
     assert n.is_alive()
-    yield [n.kill() for i in range(5)]
-    yield [n.kill() for i in range(5)]
-    yield n.close()
+    await asyncio.gather(*(n.kill() for _ in range(5)))
+    await asyncio.gather(*(n.kill() for _ in range(5)))
+    await n.close()
 
 
 @gen_cluster(Worker=Nanny)
-def test_str(s, a, b):
+async def test_str(s, a, b):
     assert a.worker_address in str(a)
     assert a.worker_address in repr(a)
     assert str(a.nthreads) in str(a)
@@ -77,59 +77,59 @@ def test_str(s, a, b):
 
 
 @gen_cluster(nthreads=[], timeout=20, client=True)
-def test_nanny_process_failure(c, s):
-    n = yield Nanny(s.address, nthreads=2, loop=s.loop)
+async def test_nanny_process_failure(c, s):
+    n = await Nanny(s.address, nthreads=2, loop=s.loop)
     first_dir = n.worker_dir
 
     assert os.path.exists(first_dir)
 
     original_address = n.worker_address
     ww = rpc(n.worker_address)
-    yield ww.update_data(data=valmap(dumps, {"x": 1, "y": 2}))
+    await ww.update_data(data=valmap(dumps, {"x": 1, "y": 2}))
     pid = n.pid
     assert pid is not None
     with ignoring(CommClosedError):
-        yield c.run(os._exit, 0, workers=[n.worker_address])
+        await c.run(os._exit, 0, workers=[n.worker_address])
 
     start = time()
     while n.pid == pid:  # wait while process dies and comes back
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
         assert time() - start < 5
 
     start = time()
-    yield gen.sleep(1)
+    await asyncio.sleep(1)
     while not n.is_alive():  # wait while process comes back
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
         assert time() - start < 5
 
     # assert n.worker_address != original_address  # most likely
 
     start = time()
     while n.worker_address not in s.nthreads or n.worker_dir is None:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
         assert time() - start < 5
 
     second_dir = n.worker_dir
 
-    yield n.close()
+    await n.close()
     assert not os.path.exists(second_dir)
     assert not os.path.exists(first_dir)
     assert first_dir != n.worker_dir
-    yield ww.close_rpc()
+    await ww.close_rpc()
     s.stop()
 
 
 @gen_cluster(nthreads=[])
-def test_run(s):
+async def test_run(s):
     pytest.importorskip("psutil")
-    n = yield Nanny(s.address, nthreads=2, loop=s.loop)
+    n = await Nanny(s.address, nthreads=2, loop=s.loop)
 
     with rpc(n.address) as nn:
-        response = yield nn.run(function=dumps(lambda: 1))
+        response = await nn.run(function=dumps(lambda: 1))
         assert response["status"] == "OK"
         assert response["result"] == 1
 
-    yield n.close()
+    await n.close()
 
 
 @pytest.mark.slow
@@ -150,12 +150,12 @@ async def test_no_hang_when_scheduler_closes(s, a, b):
 @gen_cluster(
     Worker=Nanny, nthreads=[("127.0.0.1", 1)], worker_kwargs={"reconnect": False}
 )
-def test_close_on_disconnect(s, w):
-    yield s.close()
+async def test_close_on_disconnect(s, w):
+    await s.close()
 
     start = time()
     while w.status != "closed":
-        yield gen.sleep(0.05)
+        await asyncio.sleep(0.05)
         assert time() < start + 9
 
 
@@ -165,70 +165,69 @@ class Something(Worker):
 
 
 @gen_cluster(client=True, Worker=Nanny)
-def test_nanny_worker_class(c, s, w1, w2):
-    out = yield c._run(lambda dask_worker=None: str(dask_worker.__class__))
+async def test_nanny_worker_class(c, s, w1, w2):
+    out = await c._run(lambda dask_worker=None: str(dask_worker.__class__))
     assert "Worker" in list(out.values())[0]
     assert w1.Worker is Worker
 
 
 @gen_cluster(client=True, Worker=Nanny, worker_kwargs={"worker_class": Something})
-def test_nanny_alt_worker_class(c, s, w1, w2):
-    out = yield c._run(lambda dask_worker=None: str(dask_worker.__class__))
+async def test_nanny_alt_worker_class(c, s, w1, w2):
+    out = await c._run(lambda dask_worker=None: str(dask_worker.__class__))
     assert "Something" in list(out.values())[0]
     assert w1.Worker is Something
 
 
 @pytest.mark.slow
 @gen_cluster(client=False, nthreads=[])
-def test_nanny_death_timeout(s):
-    yield s.close()
+async def test_nanny_death_timeout(s):
+    await s.close()
     w = Nanny(s.address, death_timeout=1)
     with pytest.raises(TimeoutError):
-        yield w
+        await w
 
     assert w.status == "closed"
 
 
 @gen_cluster(client=True, Worker=Nanny)
-def test_random_seed(c, s, a, b):
-    @gen.coroutine
-    def check_func(func):
+async def test_random_seed(c, s, a, b):
+    async def check_func(func):
         x = c.submit(func, 0, 2 ** 31, pure=False, workers=a.worker_address)
         y = c.submit(func, 0, 2 ** 31, pure=False, workers=b.worker_address)
         assert x.key != y.key
-        x = yield x
-        y = yield y
+        x = await x
+        y = await y
         assert x != y
 
-    yield check_func(lambda a, b: random.randint(a, b))
-    yield check_func(lambda a, b: np.random.randint(a, b))
+    await check_func(lambda a, b: random.randint(a, b))
+    await check_func(lambda a, b: np.random.randint(a, b))
 
 
 @pytest.mark.skipif(
     sys.platform.startswith("win"), reason="num_fds not supported on windows"
 )
 @gen_cluster(client=False, nthreads=[])
-def test_num_fds(s):
+async def test_num_fds(s):
     psutil = pytest.importorskip("psutil")
     proc = psutil.Process()
 
     # Warm up
-    w = yield Nanny(s.address)
-    yield w.close()
+    w = await Nanny(s.address)
+    await w.close()
     del w
     gc.collect()
 
     before = proc.num_fds()
 
     for i in range(3):
-        w = yield Nanny(s.address)
-        yield gen.sleep(0.1)
-        yield w.close()
+        w = await Nanny(s.address)
+        await asyncio.sleep(0.1)
+        await w.close()
 
     start = time()
     while proc.num_fds() > before:
         print("fds:", before, proc.num_fds())
-        yield gen.sleep(0.1)
+        await asyncio.sleep(0.1)
         assert time() < start + 10
 
 
@@ -236,42 +235,42 @@ def test_num_fds(s):
     not sys.platform.startswith("linux"), reason="Need 127.0.0.2 to mean localhost"
 )
 @gen_cluster(client=True, nthreads=[])
-def test_worker_uses_same_host_as_nanny(c, s):
+async def test_worker_uses_same_host_as_nanny(c, s):
     for host in ["tcp://0.0.0.0", "tcp://127.0.0.2"]:
-        n = yield Nanny(s.address, host=host)
+        n = await Nanny(s.address, host=host)
 
         def func(dask_worker):
             return dask_worker.listener.listen_address
 
-        result = yield c.run(func)
+        result = await c.run(func)
         assert host in first(result.values())
-        yield n.close()
+        await n.close()
 
 
 @gen_test()
-def test_scheduler_file():
+async def test_scheduler_file():
     with tmpfile() as fn:
-        s = yield Scheduler(scheduler_file=fn, port=8008)
-        w = yield Nanny(scheduler_file=fn)
+        s = await Scheduler(scheduler_file=fn, port=8008)
+        w = await Nanny(scheduler_file=fn)
         assert set(s.workers) == {w.worker_address}
-        yield w.close()
+        await w.close()
         s.stop()
 
 
 @gen_cluster(client=True, Worker=Nanny, nthreads=[("127.0.0.1", 2)])
-def test_nanny_timeout(c, s, a):
-    x = yield c.scatter(123)
+async def test_nanny_timeout(c, s, a):
+    x = await c.scatter(123)
     with captured_logger(
         logging.getLogger("distributed.nanny"), level=logging.ERROR
     ) as logger:
-        response = yield a.restart(timeout=0.1)
+        response = await a.restart(timeout=0.1)
 
     out = logger.getvalue()
     assert "timed out" in out.lower()
 
     start = time()
     while x.status != "cancelled":
-        yield gen.sleep(0.1)
+        await asyncio.sleep(0.1)
         assert time() < start + 7
 
 
@@ -283,7 +282,7 @@ def test_nanny_timeout(c, s, a):
     timeout=20,
     clean_kwargs={"threads": False},
 )
-def test_nanny_terminate(c, s, a):
+async def test_nanny_terminate(c, s, a):
     from time import sleep
 
     def leak():
@@ -297,7 +296,7 @@ def test_nanny_terminate(c, s, a):
         future = c.submit(leak)
         start = time()
         while a.process.pid == proc:
-            yield gen.sleep(0.1)
+            await asyncio.sleep(0.1)
             assert time() < start + 10
         out = logger.getvalue()
         assert "restart" in out.lower()
@@ -339,45 +338,45 @@ async def test_throttle_outgoing_connections(c, s, a, *workers):
 
 
 @gen_cluster(nthreads=[], client=True)
-def test_avoid_memory_monitor_if_zero_limit(c, s):
-    nanny = yield Nanny(s.address, loop=s.loop, memory_limit=0)
-    typ = yield c.run(lambda dask_worker: type(dask_worker.data))
+async def test_avoid_memory_monitor_if_zero_limit(c, s):
+    nanny = await Nanny(s.address, loop=s.loop, memory_limit=0)
+    typ = await c.run(lambda dask_worker: type(dask_worker.data))
     assert typ == {nanny.worker_address: dict}
-    pcs = yield c.run(lambda dask_worker: list(dask_worker.periodic_callbacks))
+    pcs = await c.run(lambda dask_worker: list(dask_worker.periodic_callbacks))
     assert "memory" not in pcs
     assert "memory" not in nanny.periodic_callbacks
 
     future = c.submit(inc, 1)
-    assert (yield future) == 2
-    yield gen.sleep(0.02)
+    assert await future == 2
+    await asyncio.sleep(0.02)
 
-    yield c.submit(inc, 2)  # worker doesn't pause
+    await c.submit(inc, 2)  # worker doesn't pause
 
-    yield nanny.close()
+    await nanny.close()
 
 
 @gen_cluster(nthreads=[], client=True)
-def test_scheduler_address_config(c, s):
+async def test_scheduler_address_config(c, s):
     with dask.config.set({"scheduler-address": s.address}):
-        nanny = yield Nanny(loop=s.loop)
+        nanny = await Nanny(loop=s.loop)
         assert nanny.scheduler.address == s.address
 
         start = time()
         while not s.workers:
-            yield gen.sleep(0.1)
+            await asyncio.sleep(0.1)
             assert time() < start + 10
 
-    yield nanny.close()
+    await nanny.close()
 
 
 @pytest.mark.slow
 @gen_test(timeout=20)
-def test_wait_for_scheduler():
+async def test_wait_for_scheduler():
     with captured_logger("distributed") as log:
         w = Nanny("127.0.0.1:44737")
         IOLoop.current().add_callback(w.start)
-        yield gen.sleep(6)
-        yield w.close()
+        await asyncio.sleep(6)
+        await w.close()
 
     log = log.getvalue()
     assert "error" not in log.lower(), log
@@ -385,31 +384,31 @@ def test_wait_for_scheduler():
 
 
 @gen_cluster(nthreads=[], client=True)
-def test_environment_variable(c, s):
+async def test_environment_variable(c, s):
     a = Nanny(s.address, loop=s.loop, memory_limit=0, env={"FOO": "123"})
     b = Nanny(s.address, loop=s.loop, memory_limit=0, env={"FOO": "456"})
-    yield [a, b]
-    results = yield c.run(lambda: os.environ["FOO"])
+    await asyncio.gather(a, b)
+    results = await c.run(lambda: os.environ["FOO"])
     assert results == {a.worker_address: "123", b.worker_address: "456"}
-    yield [a.close(), b.close()]
+    await asyncio.gather(a.close(), b.close())
 
 
 @gen_cluster(nthreads=[], client=True)
-def test_data_types(c, s):
-    w = yield Nanny(s.address, data=dict)
-    r = yield c.run(lambda dask_worker: type(dask_worker.data))
+async def test_data_types(c, s):
+    w = await Nanny(s.address, data=dict)
+    r = await c.run(lambda dask_worker: type(dask_worker.data))
     assert r[w.worker_address] == dict
-    yield w.close()
+    await w.close()
 
 
 @gen_cluster(nthreads=[])
-def test_local_directory(s):
+async def test_local_directory(s):
     with tmpfile() as fn:
         with dask.config.set(temporary_directory=fn):
-            w = yield Nanny(s.address)
+            w = await Nanny(s.address)
             assert w.local_directory.startswith(fn)
             assert "dask-worker-space" in w.local_directory
-            yield w.close()
+            await w.close()
 
 
 def _noop(x):
@@ -423,13 +422,13 @@ def _noop(x):
     Worker=Nanny,
     config={"distributed.worker.daemon": False},
 )
-def test_mp_process_worker_no_daemon(c, s, a):
+async def test_mp_process_worker_no_daemon(c, s, a):
     def multiprocessing_worker():
         p = mp.Process(target=_noop, args=(None,))
         p.start()
         p.join()
 
-    yield c.submit(multiprocessing_worker)
+    await c.submit(multiprocessing_worker)
 
 
 @gen_cluster(
@@ -438,12 +437,12 @@ def test_mp_process_worker_no_daemon(c, s, a):
     Worker=Nanny,
     config={"distributed.worker.daemon": False},
 )
-def test_mp_pool_worker_no_daemon(c, s, a):
+async def test_mp_pool_worker_no_daemon(c, s, a):
     def pool_worker(world_size):
         with mp.Pool(processes=world_size) as p:
             p.map(_noop, range(world_size))
 
-    yield c.submit(pool_worker, 4)
+    await c.submit(pool_worker, 4)
 
 
 @pytest.mark.asyncio
@@ -490,7 +489,7 @@ async def test_nanny_closes_cleanly(cleanup):
                     IOLoop.current().add_callback(w.terminate)
                     start = time()
                     while n.status != "closed":
-                        await gen.sleep(0.01)
+                        await asyncio.sleep(0.01)
                         assert time() < start + 5
 
                     assert n.status == "closed"
